@@ -8,7 +8,7 @@ from dataclasses import dataclass
 import pandas as pd
 import numpy as np
 from pandas import DataFrame, Series
-from .exceptions import ResponseError
+from .exceptions import ResponseError, NoData, ResponseParseError
 from .enums import DataType, MessageType
 
 
@@ -24,7 +24,22 @@ class Header:
     size: int
 
     @classmethod
-    def parse(cls, data: bytes) -> Header:
+    def parse(cls, request: str, data: bytes) -> Header:
+        """Parse binary header data into an object.
+
+        :param request: the request that returned the header data
+        :param data: raw header data, 20 bytes long
+        :raises ResponseParseError: if parsing failed
+        """
+        try:
+            return cls._parse(data)
+        except Exception as e:
+            raise ResponseParseError(
+                f"Failed to parse header for request: {request}. Please send this error to support."
+            ) from e
+
+    @classmethod
+    def _parse(cls, data: bytes) -> Header:
         """Parse binary header data into an object.
 
         :param data: raw header data, 20 bytes long
@@ -66,12 +81,16 @@ class Header:
 def _check_body_errors(header: Header, body_data: bytes):
     """Check for errors from the Terminal.
 
+    :raises NoData: if the server does not contain data for the request.
     :raises ResponseError: if the header indicates an error, containing a
                            helpful error message.
     """
     if header.message_type == MessageType.ERROR:
         msg = body_data.decode("ascii")
-        raise ResponseError(msg)
+        if "no data" in msg.lower():
+            raise NoData(msg)
+        else:
+            raise ResponseError(msg)
 
 
 # map price types to price multipliers
@@ -105,41 +124,49 @@ _to_price_mul = np.vectorize(lambda pt: _pt_to_price_mul[pt], otypes=[float])
 class TickBody:
     """Represents the body returned on Terminal calls that deal with ticks."""
 
-    def __init__(self, ticks: DataFrame):
-        assert isinstance(
-            ticks, DataFrame
+    def __init__(self, format_tick: list[DataType], body_ticks: np.ndarray):
+        assert isinstance(format_tick, list) and isinstance(
+            body_ticks, np.ndarray
         ), "Cannot initialize body bc ticks is not a DataFrame"
-        self.ticks: DataFrame = ticks
+        self.format_tick: list[DataType] = format_tick
+        self.body_ticks: np.ndarray = body_ticks
 
     @classmethod
-    def parse(cls, header: Header, data: bytearray) -> TickBody:
-        """Parse binary body data into an object.
+    def parse(cls, request: str, header: Header, data: bytearray) -> DataFrame:
+        """Efficiently parse binary tick data.
 
+        :param request: the request that returned the body data
         :param header: parsed header data
         :param data: the binary response body
+        :return: a processed pandas dataframe
+        :raises ResponseParseError: if parsing failed
         """
-        assert (
-            len(data) == header.size
-        ), f"Cannot parse body with {len(data)} bytes. Expected {header.size} bytes."
         assert isinstance(
             data, bytearray
         ), f"Expected data to be bytearray type. Got {type(data)}"
         _check_body_errors(header, data)
+        try:
+            tbody = cls._parse(header, data)
+            df = tbody._to_dataframe()
+            return df
+        except Exception as e:
+            raise ResponseParseError(
+                f"Failed to parse body for request: {request}. Please send this error to support."
+            ) from e
 
+    @classmethod
+    def _parse(cls, header: Header, data: bytearray) -> TickBody:
+        assert (
+            len(data) == header.size
+        ), f"Cannot parse body with {len(data)} bytes. Expected {header.size} bytes."
         n_cols = header.format_len
         n_ticks = int(header.size / (header.format_len * 4))
 
         # parse format tick
-        format_tick_codes = []
+        format: list[DataType] = []
         for ci in range(n_cols):
             int_ = int.from_bytes(data[ci * 4 : ci * 4 + 4], "big")
-            format_tick_codes.append(int_)
-        format: list[DataType] = list(
-            map(lambda code: DataType.from_code(code), format_tick_codes)
-        )
-
-        # initialize empty dataframe w/ format columns
-        df = pd.DataFrame(columns=format)
+            format.append(DataType.from_code(int_))
 
         # parse the rest of the ticks
         # 4 byte integers w/ big endian order
@@ -151,11 +178,20 @@ class TickBody:
             .newbyteorder()  # ^^
         )
 
-        # load ticks into DataFrame
-        df = pd.DataFrame(ticks, columns=df.columns, copy=False)
-        cls._post_process(df)
+        return cls(format_tick=format, body_ticks=ticks)
 
-        return cls(ticks=df)
+    def _to_dataframe(self) -> DataFrame:
+        """Load this tick data into a pandas DataFrame and post process.
+
+        Post-processing modifies the columns of data w/ various quality-of-life
+        improvements.
+        """
+        # load ticks into DataFrame
+        df = pd.DataFrame(
+            self.body_ticks, columns=self.format_tick, copy=False
+        )
+        self._post_process(df)
+        return df
 
     @classmethod
     def _post_process(cls, df: DataFrame) -> None:
@@ -172,6 +208,7 @@ class TickBody:
 
         if DataType.PRICE_TYPE in df.columns:
             # replace price type column with price multipliers
+
             df[DataType.PRICE_TYPE] = _to_price_mul(df[DataType.PRICE_TYPE])
 
             # multiply prices by price multipliers
@@ -199,18 +236,37 @@ class ListBody:
         self.lst: Series = lst
 
     @classmethod
-    def parse(cls, header: Header, data: bytes) -> ListBody:
+    def parse(
+        cls, request: str, header: Header, data: bytes, dates: bool = False
+    ) -> ListBody:
         """Parse binary body data into an object.
 
+        :param request: the request that returned the header data
         :param header: parsed header data
         :param data: the binary response body
+        :param dates: whether to parse the data as date objects
+        :raises ResponseParseError: if parsing failed
         """
+        _check_body_errors(header, data)
+        try:
+            return cls._parse(header, data, dates)
+        except Exception as e:
+            raise ResponseParseError(
+                f"Failed to parse header for request: {request}. Please send this error to support."
+            ) from e
+
+    @classmethod
+    def _parse(
+        cls, header: Header, data: bytes, dates: bool = False
+    ) -> ListBody:
         assert (
             len(data) == header.size
         ), f"Cannot parse body with {len(data)} bytes. Expected {header.size} bytes."
-        _check_body_errors(header, data)
 
         lst = data.decode("ascii").split(",")
         lst = pd.Series(lst, copy=False)
 
+        if dates:
+            lst = pd.to_datetime(lst, format="%Y%m%d")
+
         return cls(lst=lst)
